@@ -10,13 +10,17 @@ import audeer
 import audformat
 
 from audb.core import define
+from audb.core.cache import (
+    database_cache_root,
+    default_cache_root,
+)
 from audb.core.config import config
 from audb.core.dependencies import Dependencies
 from audb.core.flavor import Flavor
+from audb.core.lock import FolderLock
 from audb.core.repository import Repository
 from audb.core.utils import (
     lookup_backend,
-    mix_mapping,
     _lookup,
 )
 
@@ -38,7 +42,7 @@ def available(
         >>> audb.available(only_latest=True)
                    backend                                    host   repository version
         name
-        emodb  artifactory  https://audeering.jfrog.io/artifactory  data-public   1.1.1
+        emodb  artifactory  https://audeering.jfrog.io/artifactory  data-public   1.2.0
 
     """  # noqa: E501
     databases = []
@@ -122,7 +126,7 @@ def cached(
     Example:
         >>> db = audb.load(
         ...     'emodb',
-        ...     version='1.1.1',
+        ...     version='1.2.0',
         ...     only_metadata=True,
         ...     full_path=False,
         ...     verbose=False,
@@ -131,7 +135,7 @@ def cached(
         >>> print(df.iloc[0].to_string())
         name                emodb
         flavor_id        d3b62a9b
-        version             1.1.1
+        version             1.2.0
         complete            False
         bit_depth            None
         channels             None
@@ -140,7 +144,7 @@ def cached(
         sampling_rate        None
 
     """  # noqa: E501
-    cache_root = audeer.safe_path(
+    cache_root = audeer.path(
         cache_root or default_cache_root(shared=shared)
     )
 
@@ -221,49 +225,12 @@ def cached(
     return df.where(pd.notnull(df), None)
 
 
-def default_cache_root(
-        shared=False,
-) -> str:
-    r"""Default cache folder.
-
-    If ``shared`` is ``True``,
-    returns the path specified
-    by the environment variable
-    ``AUDB_SHARED_CACHE_ROOT``
-    or
-    ``audb.config.SHARED_CACHE_ROOT``.
-    If ``shared`` is ``False``,
-    returns the path specified
-    by the environment variable
-    ``AUDB_CACHE_ROOT``
-    or
-    ``audb.config.CACHE_ROOT``.
-
-    Args:
-        shared: if ``True`` returns path to shared cache folder
-
-    Returns:
-        path normalized by :func:`audeer.safe_path`
-
-    """
-    if shared:
-        cache = (
-            os.environ.get('AUDB_SHARED_CACHE_ROOT')
-            or config.SHARED_CACHE_ROOT
-        )
-    else:
-        cache = (
-            os.environ.get('AUDB_CACHE_ROOT')
-            or config.CACHE_ROOT
-        )
-    return audeer.safe_path(cache)
-
-
 def dependencies(
         name: str,
         *,
         version: str = None,
         cache_root: str = None,
+        verbose: bool = False,
 ) -> Dependencies:
     r"""Database dependencies.
 
@@ -272,12 +239,13 @@ def dependencies(
         version: version of database
         cache_root: cache folder where databases are stored.
             If not set :meth:`audb.default_cache_root` is used
+        verbose: show debug messages
 
     Returns:
         dependency object
 
     Example:
-        >>> deps = dependencies('emodb', version='1.1.1')
+        >>> deps = dependencies('emodb', version='1.2.0')
         >>> deps.version('db.emotion.csv')
         '1.1.0'
 
@@ -285,38 +253,31 @@ def dependencies(
     if version is None:
         version = latest_version(name)
 
-    cache_roots = [
-        default_cache_root(True),  # check shared cache first
-        default_cache_root(False),
-    ] if cache_root is None else [cache_root]
-    for cache_root in cache_roots:
-        deps_root = audeer.safe_path(
-            os.path.join(
-                cache_root,
-                name,
-                version,
-            )
-        )
-        if os.path.exists(deps_root):
-            break
-
-    audeer.mkdir(deps_root)
-    deps_path = os.path.join(deps_root, define.CACHED_DEPENDENCIES_FILE)
+    db_root = database_cache_root(
+        name,
+        version,
+        cache_root=cache_root,
+    )
+    deps_path = os.path.join(db_root, define.CACHED_DEPENDENCIES_FILE)
 
     deps = Dependencies()
-    try:
-        deps.load(deps_path)
-    except (FileNotFoundError, ValueError, EOFError):
-        backend = lookup_backend(name, version)
-        with tempfile.TemporaryDirectory() as tmp_root:
-            archive = backend.join(name, define.DB)
-            backend.get_archive(
-                archive,
-                tmp_root,
-                version,
-            )
-            deps.load(os.path.join(tmp_root, define.DEPENDENCIES_FILE))
-            deps.save(deps_path)
+
+    with FolderLock(db_root):
+        try:
+            deps.load(deps_path)
+        except (AttributeError, FileNotFoundError, ValueError, EOFError):
+            # If loading pickled cached file fails, load again from backend
+            backend = lookup_backend(name, version)
+            with tempfile.TemporaryDirectory() as tmp_root:
+                archive = backend.join(name, define.DB)
+                backend.get_archive(
+                    archive,
+                    tmp_root,
+                    version,
+                    verbose=verbose,
+                )
+                deps.load(os.path.join(tmp_root, define.DEPENDENCIES_FILE))
+                deps.save(deps_path)
 
     return deps
 
@@ -331,7 +292,6 @@ def exists(
     mixdown: bool = False,
     sampling_rate: int = None,
     cache_root: str = None,
-    **kwargs,
 ) -> bool:
     r"""Check if specified database flavor exists in local cache folder.
 
@@ -364,25 +324,16 @@ def exists(
     Example:
         >>> db = audb.load(
         ...     'emodb',
-        ...     version='1.1.1',
+        ...     version='1.2.0',
         ...     only_metadata=True,
         ...     verbose=False,
         ... )
-        >>> audb.exists('emodb', version='1.1.1')
+        >>> audb.exists('emodb', version='1.2.0')
         True
-        >>> audb.exists('emodb', version='1.1.1', format='wav')
+        >>> audb.exists('emodb', version='1.2.0', format='wav')
         False
 
     """
-    # Map mix to channels and mixdown
-    if (
-            channels is None
-            and not mixdown
-            and 'mix' in kwargs
-    ):  # pragma: no cover
-        mix = kwargs['mix']
-        channels, mixdown = mix_mapping(mix)
-
     if version is None:
         version = latest_version(name)
 
@@ -401,9 +352,7 @@ def exists(
         default_cache_root(False),
     ] if cache_root is None else [cache_root]
     for cache_root in cache_roots:
-        db_root = audeer.safe_path(
-            os.path.join(cache_root, relative_flavor_path)
-        )
+        db_root = audeer.path(cache_root, relative_flavor_path)
         if os.path.exists(db_root):
             return True
 
@@ -450,9 +399,15 @@ def flavor_path(
     Returns:
         flavor path relative to cache folder
 
+    Raises:
+        ValueError: if a non-supported ``bit_depth``,
+            ``format``,
+            or ``sampling_rate``
+            is requested
+
     Example:
-        >>> flavor_path('emodb', version='1.1.1')
-        'emodb/1.1.1/d3b62a9b'
+        >>> flavor_path('emodb', version='1.2.0').split(os.path.sep)
+        ['emodb', '1.2.0', 'd3b62a9b']
 
     """
     flavor = Flavor(
@@ -477,9 +432,12 @@ def latest_version(
     Returns:
         version string
 
+    Raises:
+        RuntimeError: if no version exists for the requested database
+
     Example:
         >>> latest_version('emodb')
-        '1.1.1'
+        '1.2.0'
 
     """
     vs = versions(name)
@@ -524,13 +482,18 @@ def remove_media(
                 archive,
                 db_root,
                 version,
+                verbose=verbose,
             )[0]
             deps_path = os.path.join(db_root, deps_path)
             deps = Dependencies()
             deps.load(deps_path)
             upload = False
 
-            for file in files:
+            for file in audeer.progress_bar(
+                files,
+                disable=not verbose,
+                desc=f'Remove media from v{version}',
+            ):
                 if file in deps.media:
                     archive = deps.archive(file)
 
@@ -575,6 +538,7 @@ def remove_media(
                     define.DEPENDENCIES_FILE,
                     remote_archive,
                     version,
+                    verbose=verbose,
                 )
 
 
@@ -600,7 +564,7 @@ def repository(
         RuntimeError: if database is not found
 
     Example:
-        >>> audb.repository('emodb', '1.1.1')
+        >>> audb.repository('emodb', '1.2.0')
         Repository('data-public', 'https://audeering.jfrog.io/artifactory', 'artifactory')
 
     """  # noqa: E501
@@ -620,7 +584,7 @@ def versions(
 
     Example:
         >>> versions('emodb')
-        ['1.1.0', '1.1.1']
+        ['1.1.0', '1.1.1', '1.2.0']
 
     """
     vs = []
