@@ -13,6 +13,11 @@ HEADER_FILE = f'{DB}.yaml'
 DEPENDENCIES_FILE = f'{DB}.csv'
 CACHED_DEPENDENCIES_FILE = f'{DB}.pkl'
 
+# Cache lock
+CACHED_VERSIONS_TIMEOUT = 10  # Timeout to acquire access to cached versions
+LOCK_FILE = '.lock'
+TIMEOUT_MSG = 'Lock could not be acquired. Timeout exceeded.'
+
 
 class DependField:
     r"""Fields stored in dependency table."""
