@@ -7,12 +7,15 @@ import audformat
 from audb.core import define
 from audb.core.api import (
     dependencies,
+    database_cache_root,
     latest_version,
 )
 from audb.core.load import (
-    database_cache_folder,
+    filtered_dependencies,
     load_header,
+    load_table,
 )
+from audb.core.lock import FolderLock
 
 
 def author(
@@ -33,7 +36,7 @@ def author(
         author(s) of database
 
     Example:
-        >>> author('emodb', version='1.1.1')
+        >>> author('emodb', version='1.2.0')
         'Felix Burkhardt, Astrid Paeschke, Miriam Rolfes, Walter Sendlmeier, Benjamin Weiss'
 
     """  # noqa: E501
@@ -45,6 +48,8 @@ def bit_depths(
         name: str,
         *,
         version: str = None,
+        tables: typing.Sequence = None,
+        media: typing.Sequence = None,
         cache_root: str = None,
 ) -> typing.Set[int]:
     """Media bit depth.
@@ -52,19 +57,26 @@ def bit_depths(
     Args:
         name: name of database
         version: version of database
+        tables: include only tables matching the regular expression or
+            provided in the list
+        media: include only media matching the regular expression or
+            provided in the list
         cache_root: cache folder where databases are stored.
             If not set :meth:`audb.default_cache_root` is used
 
     Returns:
         bit depths
 
+    Raises:
+        ValueError: if table or media is requested
+            that is not part of the database
+
     Example:
-        >>> bit_depths('emodb', version='1.1.1')
+        >>> bit_depths('emodb', version='1.2.0')
         {16}
 
     """
-    deps = dependencies(name, version=version, cache_root=cache_root)
-    df = deps()
+    df = filtered_dependencies(name, version, media, tables, cache_root)
     return set(df[df.type == define.DependType.MEDIA].bit_depth)
 
 
@@ -72,6 +84,8 @@ def channels(
         name: str,
         *,
         version: str = None,
+        tables: typing.Sequence = None,
+        media: typing.Sequence = None,
         cache_root: str = None,
 ) -> typing.Set[int]:
     """Media channels.
@@ -79,19 +93,26 @@ def channels(
     Args:
         name: name of database
         version: version of database
+        tables: include only tables matching the regular expression or
+            provided in the list
+        media: include only media matching the regular expression or
+            provided in the list
         cache_root: cache folder where databases are stored.
             If not set :meth:`audb.default_cache_root` is used
 
     Returns:
         channel numbers
 
+    Raises:
+        ValueError: if table or media is requested
+            that is not part of the database
+
     Example:
-        >>> channels('emodb', version='1.1.1')
+        >>> channels('emodb', version='1.2.0')
         {1}
 
     """
-    deps = dependencies(name, version=version, cache_root=cache_root)
-    df = deps()
+    df = filtered_dependencies(name, version, media, tables, cache_root)
     return set(df[df.type == define.DependType.MEDIA].channels)
 
 
@@ -113,7 +134,7 @@ def description(
         description of database
 
     Example:
-        >>> desc = description('emodb', version='1.1.1')
+        >>> desc = description('emodb', version='1.2.0')
         >>> desc.split('.')[0]  # show first sentence
         'Berlin Database of Emotional Speech'
 
@@ -126,6 +147,8 @@ def duration(
         name: str,
         *,
         version: str = None,
+        tables: typing.Sequence = None,
+        media: typing.Sequence = None,
         cache_root: str = None,
 ) -> pd.Timedelta:
     """Total media duration.
@@ -133,32 +156,41 @@ def duration(
     Args:
         name: name of database
         version: version of database
+        tables: include only tables matching the regular expression or
+            provided in the list
+        media: include only media matching the regular expression or
+            provided in the list
         cache_root: cache folder where databases are stored.
             If not set :meth:`audb.default_cache_root` is used
 
     Returns:
         duration
 
+    Raises:
+        ValueError: if table or media is requested
+            that is not part of the database
+
     Example:
-        >>> duration('emodb', version='1.1.1')
+        >>> duration('emodb', version='1.2.0')
         Timedelta('0 days 00:24:47.092187500')
+        >>> duration('emodb', version='1.2.0', media=['wav/03a01Fa.wav'])
+        Timedelta('0 days 00:00:01.898250')
 
     """
-    deps = dependencies(name, version=version, cache_root=cache_root)
-    df = deps()
+    df = filtered_dependencies(name, version, media, tables, cache_root)
     return pd.to_timedelta(
         df[df.type == define.DependType.MEDIA].duration.sum(),
         unit='s',
     )
 
 
-def formats(
+def files(
         name: str,
         *,
         version: str = None,
         cache_root: str = None,
-) -> typing.Set[str]:
-    """Media formats.
+) -> typing.List[str]:
+    """Media files included in the database.
 
     Args:
         name: name of database
@@ -167,15 +199,50 @@ def formats(
             If not set :meth:`audb.default_cache_root` is used
 
     Returns:
-        format
+        media files
 
     Example:
-        >>> formats('emodb', version='1.1.1')
-        {'wav'}
+        >>> files('emodb', version='1.2.0')[:2]
+        ['wav/03a01Fa.wav', 'wav/03a01Nc.wav']
 
     """
     deps = dependencies(name, version=version, cache_root=cache_root)
-    df = deps()
+    return deps.media
+
+
+def formats(
+        name: str,
+        *,
+        version: str = None,
+        tables: typing.Sequence = None,
+        media: typing.Sequence = None,
+        cache_root: str = None,
+) -> typing.Set[str]:
+    """Media formats.
+
+    Args:
+        name: name of database
+        version: version of database
+        tables: include only tables matching the regular expression or
+            provided in the list
+        media: include only media matching the regular expression or
+            provided in the list
+        cache_root: cache folder where databases are stored.
+            If not set :meth:`audb.default_cache_root` is used
+
+    Returns:
+        formats
+
+    Raises:
+        ValueError: if table or media is requested
+            that is not part of the database
+
+    Example:
+        >>> formats('emodb', version='1.2.0')
+        {'wav'}
+
+    """
+    df = filtered_dependencies(name, version, media, tables, cache_root)
     return set(df[df.type == define.DependType.MEDIA].format)
 
 
@@ -197,15 +264,19 @@ def header(
         database object without table data
 
     Example:
-        >>> db = header('emodb', version='1.1.1')
+        >>> db = header('emodb', version='1.2.0')
         >>> db.name
         'emodb'
 
     """
     if version is None:
         version = latest_version(name)
-    db_root = database_cache_folder(name, version, cache_root)
-    db, _ = load_header(db_root, name, version)
+
+    db_root = database_cache_root(name, version, cache_root)
+
+    with FolderLock(db_root):
+        db, _ = load_header(db_root, name, version)
+
     return db
 
 
@@ -227,7 +298,7 @@ def languages(
         languages of database
 
     Example:
-        >>> languages('emodb', version='1.1.1')
+        >>> languages('emodb', version='1.2.0')
         ['deu']
 
     """
@@ -253,7 +324,7 @@ def license(
         license of database
 
     Example:
-        >>> license('emodb', version='1.1.1')
+        >>> license('emodb', version='1.2.0')
         'CC0-1.0'
 
     """
@@ -279,7 +350,7 @@ def license_url(
         license URL of database
 
     Example:
-        >>> license_url('emodb', version='1.1.1')
+        >>> license_url('emodb', version='1.2.0')
         'https://creativecommons.org/publicdomain/zero/1.0/'
 
     """
@@ -305,7 +376,7 @@ def media(
         media of database
 
     Example:
-        >>> media('emodb', version='1.1.1')
+        >>> media('emodb', version='1.2.0')
         microphone:
             {type: other, format: wav, channels: 1, sampling_rate: 16000}
 
@@ -332,7 +403,7 @@ def meta(
         meta information of database
 
     Example:
-        >>> meta('emodb', version='1.1.1')
+        >>> meta('emodb', version='1.2.0')
         pdf:
           http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.130.8506&rep=rep1&type=pdf
 
@@ -359,7 +430,7 @@ def organization(
         organization responsible for database
 
     Example:
-        >>> organization('emodb', version='1.1.1')
+        >>> organization('emodb', version='1.2.0')
         'audEERING'
 
     """
@@ -385,7 +456,7 @@ def raters(
         raters of database
 
     Example:
-        >>> raters('emodb', version='1.1.1')
+        >>> raters('emodb', version='1.2.0')
         gold:
             {type: human}
 
@@ -398,6 +469,8 @@ def sampling_rates(
         name: str,
         *,
         version: str = None,
+        tables: typing.Sequence = None,
+        media: typing.Sequence = None,
         cache_root: str = None,
 ) -> typing.Set[int]:
     """Media sampling rates.
@@ -405,19 +478,26 @@ def sampling_rates(
     Args:
         name: name of database
         version: version of database
+        tables: include only tables matching the regular expression or
+            provided in the list
+        media: include only media matching the regular expression or
+            provided in the list
         cache_root: cache folder where databases are stored.
             If not set :meth:`audb.default_cache_root` is used
 
     Returns:
         sampling rates
 
+    Raises:
+        ValueError: if table or media is requested
+            that is not part of the database
+
     Example:
-        >>> sampling_rates('emodb', version='1.1.1')
+        >>> sampling_rates('emodb', version='1.2.0')
         {16000}
 
     """
-    deps = dependencies(name, version=version, cache_root=cache_root)
-    df = deps()
+    df = filtered_dependencies(name, version, media, tables, cache_root)
     return set(df[df.type == define.DependType.MEDIA].sampling_rate)
 
 
@@ -439,7 +519,7 @@ def schemes(
         schemes of database
 
     Example:
-        >>> list(schemes('emodb', version='1.1.1'))
+        >>> list(schemes('emodb', version='1.2.0'))
         ['confidence', 'duration', 'emotion', 'speaker', 'transcription']
 
     """
@@ -465,7 +545,7 @@ def source(
         source of database
 
     Example:
-        >>> source('emodb', version='1.1.1')
+        >>> source('emodb', version='1.2.0')
         'http://emodb.bilderbar.info/download/download.zip'
 
     """
@@ -491,10 +571,13 @@ def splits(
         splits of database
 
     Example:
-        >>> splits('emodb', version='1.1.1')
+        >>> splits('emodb', version='1.2.0')
+        test:
+          {description: Unofficial speaker-independent test split, type: test}
+        train:
+          {description: Unofficial speaker-independent train split, type: train}
 
-
-    """
+    """  # noqa: E501
     db = header(name, version=version, cache_root=cache_root)
     return db.splits
 
@@ -517,10 +600,10 @@ def tables(
         tables of database
 
     Example:
-        >>> list(tables('emodb', version='1.1.1'))
-        ['emotion', 'files']
+        >>> list(tables('emodb', version='1.2.0'))
+        ['emotion', 'emotion.categories.test.gold_standard', 'emotion.categories.train.gold_standard', 'files']
 
-    """
+    """  # noqa: E501
     db = header(name, version=version, cache_root=cache_root)
     return db.tables
 
@@ -543,7 +626,7 @@ def usage(
         usage of database
 
     Example:
-        >>> usage('emodb', version='1.1.1')
+        >>> usage('emodb', version='1.2.0')
         'unrestricted'
 
     """
