@@ -1,9 +1,9 @@
-from distutils.version import LooseVersion
 import os
 import re
 import shutil
 import typing
 
+import filelock
 import pandas as pd
 
 import audbackend
@@ -11,24 +11,39 @@ import audeer
 import audformat
 
 from audb.core import define
+from audb.core import utils
 from audb.core.api import (
     cached,
-    default_cache_root,
     dependencies,
     latest_version,
 )
-from audb.core.backward import parse_deprecated_load_arguments
-from audb.core.dependencies import Dependencies
+from audb.core.cache import (
+    database_cache_root,
+    database_tmp_root,
+    default_cache_root,
+)
+from audb.core.dependencies import (
+    Dependencies,
+    error_message_missing_object,
+    filter_media,
+    filter_tables,
+)
 from audb.core.flavor import Flavor
+from audb.core.lock import FolderLock
 from audb.core.utils import lookup_backend
 
 
+CachedVersions = typing.Sequence[
+    typing.Tuple[audeer.LooseVersion, str, Dependencies],
+]
+
+
 def _cached_versions(
         name: str,
         version: str,
         flavor: Flavor,
         cache_root: typing.Optional[str],
-) -> typing.Sequence[typing.Tuple[LooseVersion, str, Dependencies]]:
+) -> CachedVersions:
     r"""Find other cached versions of same flavor."""
 
     df = cached(cache_root=cache_root, name=name)
@@ -53,8 +68,8 @@ def _cached_versions(
             cached_versions.insert(
                 0,
                 (
-                    LooseVersion(row['version']),
-                    flavor_root,
+                    audeer.LooseVersion(row['version']),
+                    str(flavor_root),
                     deps,
                 ),
             )
@@ -65,9 +80,7 @@ def _cached_versions(
 def _cached_files(
         files: typing.Sequence[str],
         deps: Dependencies,
-        cached_versions: typing.Sequence[
-            typing.Tuple[LooseVersion, str, Dependencies],
-        ],
+        cached_versions: CachedVersions,
         flavor: typing.Optional[Flavor],
         verbose: bool,
 ) -> (typing.Sequence[typing.Union[str, str]], typing.Sequence[str]):
@@ -82,7 +95,7 @@ def _cached_files(
             disable=not verbose,
     ):
         found = False
-        file_version = LooseVersion(deps.version(file))
+        file_version = audeer.LooseVersion(deps.version(file))
         for cache_version, cache_root, cache_deps in cached_versions:
             if cache_version >= file_version:
                 if file in cache_deps:
@@ -122,13 +135,12 @@ def _copy_file(
     audeer.mkdir(os.path.dirname(tmp_path))
     audeer.mkdir(os.path.dirname(dst_path))
     shutil.copy(src_path, tmp_path)
-    _move_file(root_tmp, root_dst, file)
+    audeer.move_file(tmp_path, dst_path)
 
 
 def _database_check_complete(
         db: audformat.Database,
         db_root: str,
-        db_root_tmp: str,
         flavor: Flavor,
         deps: Dependencies,
 ):
@@ -146,11 +158,16 @@ def _database_check_complete(
         return complete
 
     if check():
+        db_root_tmp = database_tmp_root(db_root)
         db.meta['audb']['complete'] = True
         db_original = audformat.Database.load(db_root, load_data=False)
         db_original.meta['audb']['complete'] = True
         db_original.save(db_root_tmp, header_only=True)
-        _move_file(db_root_tmp, db_root, define.HEADER_FILE)
+        audeer.move_file(
+            os.path.join(db_root_tmp, define.HEADER_FILE),
+            os.path.join(db_root, define.HEADER_FILE),
+        )
+        audeer.rmdir(db_root_tmp)
 
 
 def _database_is_complete(
@@ -170,12 +187,18 @@ def _files_duration(
         format: typing.Optional[str],
 ):
     field = define.DEPEND_FIELD_NAMES[define.DependField.DURATION]
-    durs = deps._df.loc[files][field]
+    durs = deps().loc[files][field]
     durs = durs[durs > 0]
     durs = pd.to_timedelta(durs, unit='s')
     durs.index.name = 'file'
     if format is not None:
         durs.index = audformat.utils.replace_file_extension(durs.index, format)
+    # Norm file path under Windows to include `\`
+    if os.name == 'nt':  # pragma: nocover as tested in Windows runner
+        durs.index = audformat.utils.map_file_path(
+            durs.index,
+            os.path.normpath,
+        )
     durs.index = audformat.utils.expand_file_path(durs.index, db.root)
     db._files_duration = durs.to_dict()
 
@@ -184,7 +207,6 @@ def _get_media_from_backend(
         name: str,
         media: typing.Sequence[str],
         db_root: str,
-        db_root_tmp: str,
         flavor: typing.Optional[Flavor],
         deps: Dependencies,
         backend: audbackend.Backend,
@@ -214,9 +236,9 @@ def _get_media_from_backend(
     # create folder tree to avoid race condition
     # in os.makedirs when files are unpacked
     # using multi-processing
-    for file in media:
-        audeer.mkdir(os.path.dirname(os.path.join(db_root, file)))
-        audeer.mkdir(os.path.dirname(os.path.join(db_root_tmp, file)))
+    db_root_tmp = database_tmp_root(db_root)
+    utils.mkdir_tree(media, db_root)
+    utils.mkdir_tree(media, db_root_tmp)
 
     def job(archive: str, version: str):
         archive = backend.join(
@@ -245,7 +267,10 @@ def _get_media_from_backend(
                 if src_path != dst_path:
                     os.remove(src_path)
 
-            _move_file(db_root_tmp, db_root, file)
+            audeer.move_file(
+                os.path.join(db_root_tmp, file),
+                os.path.join(db_root, file),
+            )
 
     audeer.run_tasks(
         job,
@@ -255,39 +280,52 @@ def _get_media_from_backend(
         task_description='Load media',
     )
 
+    audeer.rmdir(db_root_tmp)
+
 
 def _get_media_from_cache(
         media: typing.Sequence[str],
         db_root: str,
-        db_root_tmp: str,
         deps: Dependencies,
-        cached_versions: typing.Sequence[
-            typing.Tuple[LooseVersion, str, Dependencies]
-        ],
+        cached_versions: CachedVersions,
         flavor: Flavor,
         num_workers: int,
         verbose: bool,
 ) -> typing.Sequence[str]:
     r"""Copy media from cache."""
 
-    cached_media, missing_media = _cached_files(
-        media,
-        deps,
-        cached_versions,
-        flavor,
-        verbose,
-    )
+    db_root_cached = [x[1] for x in cached_versions]
 
-    def job(cache_root: str, file: str):
-        _copy_file(file, cache_root, db_root_tmp, db_root)
+    try:
+        with FolderLock(
+                db_root_cached,
+                timeout=define.CACHED_VERSIONS_TIMEOUT,
+        ):
 
-    audeer.run_tasks(
-        job,
-        params=[([root, file], {}) for root, file in cached_media],
-        num_workers=num_workers,
-        progress_bar=verbose,
-        task_description='Copy media',
-    )
+            cached_media, missing_media = _cached_files(
+                media,
+                deps,
+                cached_versions,
+                flavor,
+                verbose,
+            )
+            db_root_tmp = database_tmp_root(db_root)
+
+            def job(cache_root: str, file: str):
+                _copy_file(file, cache_root, db_root_tmp, db_root)
+
+            audeer.run_tasks(
+                job,
+                params=[([root, file], {}) for root, file in cached_media],
+                num_workers=num_workers,
+                progress_bar=verbose,
+                task_description='Copy media',
+            )
+
+            audeer.rmdir(db_root_tmp)
+
+    except filelock.Timeout:
+        missing_media = media
 
     return missing_media
 
@@ -296,13 +334,13 @@ def _get_tables_from_backend(
         db: audformat.Database,
         tables: typing.Sequence[str],
         db_root: str,
-        db_root_tmp: str,
         deps: Dependencies,
         backend: audbackend.Backend,
         num_workers: typing.Optional[int],
         verbose: bool,
 ):
     r"""Load tables from backend."""
+    db_root_tmp = database_tmp_root(db_root)
 
     def job(table: str):
         archive = backend.join(
@@ -326,7 +364,11 @@ def _get_tables_from_backend(
             audformat.define.TableStorageFormat.PICKLE,
             audformat.define.TableStorageFormat.CSV,
         ]:
-            _move_file(db_root_tmp, db_root, f'db.{table_id}.{storage_format}')
+            file = f'db.{table_id}.{storage_format}'
+            audeer.move_file(
+                os.path.join(db_root_tmp, file),
+                os.path.join(db_root, file),
+            )
 
     audeer.run_tasks(
         job,
@@ -336,43 +378,56 @@ def _get_tables_from_backend(
         task_description='Load tables',
     )
 
+    audeer.rmdir(db_root_tmp)
+
 
 def _get_tables_from_cache(
         tables: typing.Sequence[str],
         db_root: str,
-        db_root_tmp: str,
         deps: Dependencies,
-        cached_versions: typing.Sequence[
-            typing.Tuple[LooseVersion, str, Dependencies]
-        ],
+        cached_versions: CachedVersions,
         num_workers: int,
         verbose: bool,
 ) -> typing.Sequence[str]:
     r"""Copy tables from cache."""
 
-    cached_tables, missing_tables = _cached_files(
-        tables,
-        deps,
-        cached_versions,
-        None,
-        verbose,
-    )
+    db_root_cached = [x[1] for x in cached_versions]
 
-    def job(cache_root: str, file: str):
-        file_pkl = audeer.replace_file_extension(
-            file,
-            audformat.define.TableStorageFormat.PICKLE,
-        )
-        _copy_file(file, cache_root, db_root_tmp, db_root)
-        _copy_file(file_pkl, cache_root, db_root_tmp, db_root)
+    try:
+        with FolderLock(
+                db_root_cached,
+                timeout=define.CACHED_VERSIONS_TIMEOUT,
+        ):
 
-    audeer.run_tasks(
-        job,
-        params=[([root, file], {}) for root, file in cached_tables],
-        num_workers=num_workers,
-        progress_bar=verbose,
-        task_description='Copy tables',
-    )
+            cached_tables, missing_tables = _cached_files(
+                tables,
+                deps,
+                cached_versions,
+                None,
+                verbose,
+            )
+            db_root_tmp = database_tmp_root(db_root)
+
+            def job(cache_root: str, file: str):
+                file_pkl = audeer.replace_file_extension(
+                    file,
+                    audformat.define.TableStorageFormat.PICKLE,
+                )
+                _copy_file(file, cache_root, db_root_tmp, db_root)
+                _copy_file(file_pkl, cache_root, db_root_tmp, db_root)
+
+            audeer.run_tasks(
+                job,
+                params=[([root, file], {}) for root, file in cached_tables],
+                num_workers=num_workers,
+                progress_bar=verbose,
+                task_description='Copy tables',
+            )
+
+            audeer.rmdir(db_root_tmp)
+
+    except filelock.Timeout:
+        missing_tables = tables
 
     return missing_tables
 
@@ -381,18 +436,15 @@ def _load_media(
         media: typing.Sequence[str],
         backend: audbackend.Backend,
         db_root: str,
-        db_root_tmp: str,
         name: str,
         version: str,
-        cached_versions: typing.Optional[
-            typing.Sequence[typing.Tuple[LooseVersion, str, Dependencies]]
-        ],
+        cached_versions: typing.Optional[CachedVersions],
         deps: Dependencies,
         flavor: Flavor,
         cache_root: str,
         num_workers: int,
         verbose: bool,
-):
+) -> typing.Optional[CachedVersions]:
     r"""Load media files to cache.
 
     All media files not existing in cache yet
@@ -419,7 +471,6 @@ def _load_media(
             missing_media = _get_media_from_cache(
                 missing_media,
                 db_root,
-                db_root_tmp,
                 deps,
                 cached_versions,
                 flavor,
@@ -433,7 +484,6 @@ def _load_media(
                 name,
                 missing_media,
                 db_root,
-                db_root_tmp,
                 flavor,
                 deps,
                 backend,
@@ -441,23 +491,22 @@ def _load_media(
                 verbose,
             )
 
+    return cached_versions
+
 
 def _load_tables(
         tables: typing.Sequence[str],
         backend: audbackend.Backend,
         db_root: str,
-        db_root_tmp: str,
         db: audformat.Database,
         version: str,
-        cached_versions: typing.Optional[
-            typing.Sequence[typing.Tuple[LooseVersion, str, Dependencies]]
-        ],
+        cached_versions: typing.Optional[CachedVersions],
         deps: Dependencies,
         flavor: Flavor,
         cache_root: str,
         num_workers: int,
         verbose: bool,
-):
+) -> typing.Optional[CachedVersions]:
     r"""Load table files to cache.
 
     All table files not existing in cache yet
@@ -483,7 +532,6 @@ def _load_tables(
             missing_tables = _get_tables_from_cache(
                 missing_tables,
                 db_root,
-                db_root_tmp,
                 deps,
                 cached_versions,
                 num_workers,
@@ -496,29 +544,13 @@ def _load_tables(
                 db,
                 missing_tables,
                 db_root,
-                db_root_tmp,
                 deps,
                 backend,
                 num_workers,
                 verbose,
             )
 
-
-def _media(
-        db: audformat.Database,
-        media: typing.Optional[typing.Union[str, typing.Sequence[str]]],
-) -> typing.Sequence[str]:
-
-    if media is None:
-        media = db.files
-    elif isinstance(media, str):
-        pattern = re.compile(media)
-        media = []
-        for m in db.files:
-            if pattern.search(m):
-                media.append(m)
-
-    return media
+    return cached_versions
 
 
 def _missing_media(
@@ -559,18 +591,6 @@ def _missing_tables(
     return missing_tables
 
 
-def _move_file(
-        root_src: str,
-        root_dst: str,
-        file: str,
-):
-    r"""Move file to another directory."""
-    os.rename(
-        os.path.join(root_src, file),
-        os.path.join(root_dst, file),
-    )
-
-
 def _remove_media(
         db: audformat.Database,
         deps: Dependencies,
@@ -586,21 +606,6 @@ def _remove_media(
         )
 
 
-def _tables(
-        deps: Dependencies,
-        tables: typing.Optional[typing.Union[str, typing.Sequence[str]]],
-) -> typing.Sequence[str]:
-    if tables is None:
-        tables = deps.table_ids
-    elif isinstance(tables, str):
-        pattern = re.compile(tables)
-        tables = []
-        for table in deps.table_ids:
-            if pattern.search(table):
-                tables.append(table)
-    return tables
-
-
 def _update_path(
         db: audformat.Database,
         root: str,
@@ -629,6 +634,12 @@ def _update_path(
                 table._df.index,
                 root,
             )
+            # Norm file path under Windows to include `\`
+            if os.name == 'nt':  # pragma: nocover as tested in Windows runner
+                table._df.index = audformat.utils.map_file_path(
+                    table._df.index,
+                    os.path.normpath,
+                )
         if format is not None:
             table._df.index = audformat.utils.replace_file_extension(
                 table._df.index,
@@ -645,68 +656,55 @@ def _update_path(
     )
 
 
-def database_cache_folder(
+def filtered_dependencies(
         name: str,
         version: str,
+        media: typing.Union[str, typing.Sequence[str]],
+        tables: typing.Union[str, typing.Sequence[str]],
         cache_root: str = None,
-        flavor: Flavor = None,
-) -> str:
-    r"""Create and return database cache folder.
+) -> pd.DataFrame:
+    r"""Filter media by tables.
+
+    Return all media files from ``media``
+    that are referenced in at least one table
+    from ``tables``.
+    This will download all tables.
 
     Args:
         name: name of database
         version: version of database
-        cache_root: path to cache folder
-        flavor: flavor of database
+        media: media files
+        tables: table IDs
+        cache_root: cache folder where databases are stored.
+            If not set :meth:`audb.default_cache_root` is used
 
     Returns:
-        path to cache folder
+        filtered dependencies
 
     """
-    if cache_root is None:
-        cache_roots = [
-            default_cache_root(True),  # check shared cache first
-            default_cache_root(False),
-        ]
+    deps = dependencies(name, version=version, cache_root=cache_root)
+    if tables is None and media is None:
+        df = deps()
     else:
-        cache_roots = [cache_root]
-    for cache_root in cache_roots:
-        if flavor is None:
-            db_root = os.path.join(
-                cache_root,
+        available_media = []
+        tables = filter_tables(tables, deps.table_ids)
+        for table in tables:
+            df = load_table(
                 name,
-                version,
+                table,
+                version=version,
+                cache_root=cache_root,
+                verbose=False,
             )
-        else:
-            db_root = os.path.join(
-                cache_root,
-                flavor.path(name, version),
+            available_media += list(
+                df.index.get_level_values('file').unique()
             )
-        db_root = audeer.safe_path(db_root)
-        if os.path.exists(db_root):
-            break
 
-    audeer.mkdir(db_root)
-    return db_root
+        media = filter_media(media, deps.media, name, version)
+        available_media = [m for m in media if m in list(set(available_media))]
+        df = deps().loc[available_media]
 
-
-def database_tmp_folder(
-        cache_root: str,
-) -> str:
-    r"""Create and return temporary database cache folder.
-
-    The temporary cache folder is created under ``cache_root + '~'``.
-
-    Args:
-        cache_root: path to cache folder
-
-    Returns:
-        path to temporary cache folder
-
-    """
-    tmp_root = cache_root + '~'
-    tmp_root = audeer.mkdir(tmp_root)
-    return tmp_root
+    return df
 
 
 def load(
@@ -725,9 +723,9 @@ def load(
         full_path: bool = True,
         cache_root: str = None,
         num_workers: typing.Optional[int] = 1,
+        timeout: float = -1,
         verbose: bool = True,
-        **kwargs,
-) -> audformat.Database:
+) -> typing.Optional[audformat.Database]:
     r"""Load database.
 
     Loads meta and media files of a database to the local cache and returns
@@ -777,15 +775,28 @@ def load(
         num_workers: number of parallel jobs or 1 for sequential
             processing. If ``None`` will be set to the number of
             processors on the machine multiplied by 5
+        timeout: maximum wait time if another thread or process is already
+            accessing the database. If timeout is reached, ``None`` is
+            returned. If timeout < 0 the method will block until the
+            database can be accessed
         verbose: show debug messages
 
     Returns:
         database object
 
+    Raises:
+        ValueError: if table or media is requested
+            that is not part of the database
+        ValueError: if a non-supported ``bit_depth``,
+            ``format``,
+            or ``sampling_rate``
+            is requested
+
     Example:
         >>> db = audb.load(
         ...     'emodb',
-        ...     version='1.1.1',
+        ...     version='1.2.0',
+        ...     tables=['emotion', 'files'],
         ...     only_metadata=True,
         ...     full_path=False,
         ...     verbose=False,
@@ -796,19 +807,9 @@ def load(
     """
     if version is None:
         version = latest_version(name)
-    deps = dependencies(name, version=version, cache_root=cache_root)
-
-    # backward compatibility to audb<1.0.0
-    channels, mixdown, media = parse_deprecated_load_arguments(
-        channels,
-        mixdown,
-        media,
-        deps,
-        kwargs,
-    )
 
+    db = None
     cached_versions = None
-
     flavor = Flavor(
         channels=channels,
         format=format,
@@ -816,91 +817,115 @@ def load(
         bit_depth=bit_depth,
         sampling_rate=sampling_rate,
     )
-    db_root = database_cache_folder(name, version, cache_root, flavor)
-    db_root_tmp = database_tmp_folder(db_root)
+    db_root = database_cache_root(name, version, cache_root, flavor)
 
     if verbose:  # pragma: no cover
         print(f'Get:   {name} v{version}')
         print(f'Cache: {db_root}')
 
-    # Start with database header without tables
-    db, backend = load_header(
-        db_root,
+    deps = dependencies(
         name,
-        version,
-        flavor=flavor,
-        add_audb_meta=True,
+        version=version,
+        cache_root=cache_root,
+        verbose=verbose,
     )
 
-    db_is_complete = _database_is_complete(db)
+    try:
+        with FolderLock(db_root, timeout=timeout):
 
-    # filter tables
-    requested_tables = _tables(deps, tables)
+            # Start with database header without tables
+            db, backend = load_header(
+                db_root,
+                name,
+                version,
+                flavor=flavor,
+                add_audb_meta=True,
+            )
 
-    # load missing tables
-    if not db_is_complete:
-        _load_tables(
-            requested_tables,
-            backend,
-            db_root,
-            db_root_tmp,
-            db,
-            version,
-            cached_versions,
-            deps,
-            flavor,
-            cache_root,
-            num_workers,
-            verbose,
-        )
+            db_is_complete = _database_is_complete(db)
 
-    # filter tables
-    if tables is not None:
-        db.pick_tables(requested_tables)
+            # filter tables
+            requested_tables = filter_tables(tables, deps.table_ids)
 
-    # load tables
-    for table in requested_tables:
-        db[table].load(os.path.join(db_root, f'db.{table}'))
+            # load missing tables
+            if not db_is_complete:
+                cached_versions = _load_tables(
+                    requested_tables,
+                    backend,
+                    db_root,
+                    db,
+                    version,
+                    cached_versions,
+                    deps,
+                    flavor,
+                    cache_root,
+                    num_workers,
+                    verbose,
+                )
 
-    # filter media
-    requested_media = _media(db, media)
+            # filter tables
+            if tables is not None:
+                db.pick_tables(requested_tables)
 
-    # load missing media
-    if not db_is_complete and not only_metadata:
-        _load_media(
-            requested_media,
-            backend,
-            db_root,
-            db_root_tmp,
-            name,
-            version,
-            cached_versions,
-            deps,
-            flavor,
-            cache_root,
-            num_workers,
-            verbose,
-        )
+            # load tables
+            for table in requested_tables:
+                db[table].load(os.path.join(db_root, f'db.{table}'))
 
-    # filter media
-    if media is not None or tables is not None:
-        db.pick_files(requested_media)
+            # filter media
+            requested_media = filter_media(media, db.files, name, version)
 
-    if not removed_media:
-        _remove_media(db, deps, num_workers, verbose)
+            # load missing media
+            if not db_is_complete and not only_metadata:
+                cached_versions = _load_media(
+                    requested_media,
+                    backend,
+                    db_root,
+                    name,
+                    version,
+                    cached_versions,
+                    deps,
+                    flavor,
+                    cache_root,
+                    num_workers,
+                    verbose,
+                )
 
-    # Adjust full paths and file extensions in tables
-    _update_path(db, db_root, full_path, flavor.format, num_workers, verbose)
+            # filter media
+            if media is not None or tables is not None:
+                db.pick_files(requested_media)
 
-    # set file durations
-    _files_duration(db, deps, requested_media, flavor.format)
+            if not removed_media:
+                _remove_media(db, deps, num_workers, verbose)
 
-    # check if database is now complete
-    if not db_is_complete:
-        _database_check_complete(db, db_root, db_root_tmp, flavor, deps)
+            # Adjust full paths and file extensions in tables
+            _update_path(
+                db,
+                db_root,
+                full_path,
+                flavor.format,
+                num_workers,
+                verbose,
+            )
 
-    if os.path.exists(db_root_tmp):
-        shutil.rmtree(db_root_tmp)
+            # set file durations
+            _files_duration(
+                db,
+                deps,
+                requested_media,
+                flavor.format,
+            )
+
+            # check if database is now complete
+            if not db_is_complete:
+                _database_check_complete(
+                    db,
+                    db_root,
+                    flavor,
+                    deps,
+                )
+
+    except filelock.Timeout:
+        utils.timeout_warning()
 
     return db
 
@@ -942,7 +967,7 @@ def load_header(
         backend = lookup_backend(name, version)
         remote_header = backend.join(name, define.HEADER_FILE)
         if add_audb_meta:
-            db_root_tmp = database_tmp_folder(db_root)
+            db_root_tmp = database_tmp_root(db_root)
             local_header = os.path.join(db_root_tmp, define.HEADER_FILE)
         backend.get_file(remote_header, local_header, version)
         if add_audb_meta:
@@ -954,7 +979,12 @@ def load_header(
                 'complete': False,
             }
             db.save(db_root_tmp, header_only=True)
-            _move_file(db_root_tmp, db_root, define.HEADER_FILE)
+            audeer.move_file(
+                os.path.join(db_root_tmp, define.HEADER_FILE),
+                os.path.join(db_root, define.HEADER_FILE),
+            )
+            audeer.rmdir(db_root_tmp)
+
     return audformat.Database.load(db_root, load_data=False), backend
 
 
@@ -970,8 +1000,9 @@ def load_media(
         sampling_rate: int = None,
         cache_root: str = None,
         num_workers: typing.Optional[int] = 1,
+        timeout: float = -1,
         verbose: bool = True,
-) -> typing.List:
+) -> typing.Optional[typing.List]:
     r"""Load media file(s).
 
     If you are interested in media files
@@ -1003,6 +1034,10 @@ def load_media(
         num_workers: number of parallel jobs or 1 for sequential
             processing. If ``None`` will be set to the number of
             processors on the machine multiplied by 5
+        timeout: maximum wait time if another thread or process is already
+            accessing the database. If timeout is reached, ``None`` is
+            returned. If timeout < 0 the method will block until the
+            database can be accessed
         verbose: show debug messages
 
     Returns:
@@ -1011,18 +1046,21 @@ def load_media(
     Raises:
         ValueError: if a media file is requested
             that is not part of the database
+        ValueError: if a non-supported ``bit_depth``,
+            ``format``,
+            or ``sampling_rate``
+            is requested
 
     Example:
         >>> paths = load_media(
         ...     'emodb',
         ...     ['wav/03a01Fa.wav'],
-        ...     version='1.1.1',
+        ...     version='1.2.0',
         ...     format='flac',
         ...     verbose=False,
         ... )
-        >>> cache_root = audb.default_cache_root()
-        >>> [p[len(cache_root):] for p in paths]
-        ['/emodb/1.1.1/40bb2241/wav/03a01Fa.flac']
+        >>> paths[0].split(os.path.sep)[-5:]
+        ['emodb', '1.2.0', '40bb2241', 'wav', '03a01Fa.flac']
 
     """
     media = audeer.to_list(media)
@@ -1031,17 +1069,8 @@ def load_media(
 
     if version is None:
         version = latest_version(name)
-    deps = dependencies(name, version=version, cache_root=cache_root)
-
-    available_files = deps.media
-    for media_file in media:
-        if media_file not in available_files:
-            raise ValueError(
-                f"Could not find '{media_file}' in {name} {version}"
-            )
-
-    cached_versions = None
 
+    files = None
     flavor = Flavor(
         channels=channels,
         format=format,
@@ -1049,45 +1078,72 @@ def load_media(
         bit_depth=bit_depth,
         sampling_rate=sampling_rate,
     )
-    db_root = database_cache_folder(name, version, cache_root, flavor)
-    db_root_tmp = database_tmp_folder(db_root)
+    db_root = database_cache_root(name, version, cache_root, flavor)
 
     if verbose:  # pragma: no cover
         print(f'Get:   {name} v{version}')
         print(f'Cache: {db_root}')
 
-    # Start with database header without tables
-    db, backend = load_header(
-        db_root,
+    deps = dependencies(
         name,
-        version,
-        flavor=flavor,
-        add_audb_meta=True,
+        version=version,
+        cache_root=cache_root,
+        verbose=verbose,
     )
 
-    db_is_complete = _database_is_complete(db)
+    available_files = deps.media
+    for media_file in media:
+        if media_file not in available_files:
+            msg = error_message_missing_object(
+                'media file',
+                [media_file],
+                name,
+                version,
+            )
+            raise ValueError(msg)
 
-    # load missing media
-    if not db_is_complete:
-        _load_media(
-            media,
-            backend,
-            db_root,
-            db_root_tmp,
-            name,
-            version,
-            cached_versions,
-            deps,
-            flavor,
-            cache_root,
-            num_workers,
-            verbose,
-        )
+    try:
+        with FolderLock(db_root, timeout=timeout):
 
-    if format is not None:
-        media = [audeer.replace_file_extension(m, format) for m in media]
+            # Start with database header without tables
+            db, backend = load_header(
+                db_root,
+                name,
+                version,
+                flavor=flavor,
+                add_audb_meta=True,
+            )
 
-    return [os.path.join(db_root, m) for m in media]
+            db_is_complete = _database_is_complete(db)
+
+            # load missing media
+            if not db_is_complete:
+                _load_media(
+                    media,
+                    backend,
+                    db_root,
+                    name,
+                    version,
+                    None,
+                    deps,
+                    flavor,
+                    cache_root,
+                    num_workers,
+                    verbose,
+                )
+
+            if format is not None:
+                media = [
+                    audeer.replace_file_extension(m, format) for m in media
+                ]
+            files = [
+                os.path.join(db_root, os.path.normpath(m)) for m in media
+            ]
+
+    except filelock.Timeout:
+        utils.timeout_warning()
+
+    return files
 
 
 def load_table(
@@ -1130,7 +1186,7 @@ def load_table(
         >>> df = load_table(
         ...     'emodb',
         ...     'emotion',
-        ...     version='1.1.1',
+        ...     version='1.2.0',
         ...     verbose=False,
         ... )
         >>> df[:3]
@@ -1143,50 +1199,58 @@ def load_table(
     """
     if version is None:
         version = latest_version(name)
-    deps = dependencies(name, version=version, cache_root=cache_root)
 
-    if table not in deps.table_ids:
-        raise ValueError(
-            f"Could not find table '{table}' in {name} {version}"
-        )
-
-    cached_versions = None
-
-    db_root = database_cache_folder(name, version, cache_root)
-    db_root_tmp = database_tmp_folder(db_root)
+    db_root = database_cache_root(name, version, cache_root)
 
     if verbose:  # pragma: no cover
         print(f'Get:   {name} v{version}')
         print(f'Cache: {db_root}')
 
-    # Start with database header without tables
-    db, backend = load_header(
-        db_root,
+    deps = dependencies(
         name,
-        version,
+        version=version,
+        cache_root=cache_root,
+        verbose=verbose,
     )
 
-    # Load table
-    table_file = os.path.join(db_root, f'db.{table}')
-    if not (
-            os.path.exists(f'{table_file}.csv')
-            or os.path.exists(f'{table_file}.pkl')
-    ):
-        _load_tables(
+    if table not in deps.table_ids:
+        msg = error_message_missing_object(
+            'table',
             [table],
-            backend,
-            db_root,
-            db_root_tmp,
-            db,
+            name,
             version,
-            cached_versions,
-            deps,
-            Flavor(),
-            cache_root,
-            num_workers,
-            verbose,
         )
-    table = audformat.Table()
-    table.load(table_file)
+        raise ValueError(msg)
+
+    with FolderLock(db_root):
+
+        # Start with database header without tables
+        db, backend = load_header(
+            db_root,
+            name,
+            version,
+        )
+
+        # Load table
+        table_file = os.path.join(db_root, f'db.{table}')
+        if not (
+                os.path.exists(f'{table_file}.csv')
+                or os.path.exists(f'{table_file}.pkl')
+        ):
+            _load_tables(
+                [table],
+                backend,
+                db_root,
+                db,
+                version,
+                None,
+                deps,
+                Flavor(),
+                cache_root,
+                num_workers,
+                verbose,
+            )
+        table = audformat.Table()
+        table.load(table_file)
 
     return table._df
