@@ -7,13 +7,14 @@ import audeer
 import audformat
 
 from audb.core import define
+from audb.core import utils
 from audb.core.api import (
     dependencies,
     latest_version,
 )
 from audb.core.dependencies import Dependencies
 from audb.core.load import (
-    database_tmp_folder,
+    database_tmp_root,
     load_header,
 )
 
@@ -93,9 +94,8 @@ def _get_media(
 
     # create folder tree to avoid race condition
     # in os.makedirs when files are unpacked
-    for file in media:
-        audeer.mkdir(os.path.dirname(os.path.join(db_root, file)))
-        audeer.mkdir(os.path.dirname(os.path.join(db_root_tmp, file)))
+    utils.mkdir_tree(media, db_root)
+    utils.mkdir_tree(media, db_root_tmp)
 
     # figure out archives
     archives = set()
@@ -112,7 +112,10 @@ def _get_media(
         )
         files = backend.get_archive(archive, db_root_tmp, version)
         for file in files:
-            _move_file(db_root_tmp, db_root, file)
+            audeer.move_file(
+                os.path.join(db_root_tmp, file),
+                os.path.join(db_root, file),
+            )
 
     audeer.run_tasks(
         job,
@@ -151,7 +154,10 @@ def _get_tables(
             deps.archive(table),
         )
         backend.get_archive(archive, db_root_tmp, deps.version(table))
-        _move_file(db_root_tmp, db_root, table)
+        audeer.move_file(
+            os.path.join(db_root_tmp, table),
+            os.path.join(db_root, table),
+        )
 
     audeer.run_tasks(
         job,
@@ -162,19 +168,6 @@ def _get_tables(
     )
 
 
-def _move_file(
-        root_src: str,
-        root_dst: str,
-        file: str,
-):
-    r"""Move file to another directory."""
-
-    os.rename(
-        os.path.join(root_src, file),
-        os.path.join(root_dst, file),
-    )
-
-
 def _remove_empty_dirs(root):
     r"""Remove directories, fails if it contains non-empty sub-folders."""
 
@@ -207,12 +200,18 @@ def _save_database(
             num_workers=num_workers,
             verbose=verbose,
         )
-        _move_file(db_root_tmp, db_root, define.HEADER_FILE)
+        audeer.move_file(
+            os.path.join(db_root_tmp, define.HEADER_FILE),
+            os.path.join(db_root, define.HEADER_FILE),
+        )
         for path in glob.glob(
                 os.path.join(db_root_tmp, f'*.{storage_format}')
         ):
             file = os.path.relpath(path, db_root_tmp)
-            _move_file(db_root_tmp, db_root, file)
+            audeer.move_file(
+                os.path.join(db_root_tmp, file),
+                os.path.join(db_root, file),
+            )
 
 
 def load_to(
@@ -220,6 +219,7 @@ def load_to(
         name: str,
         *,
         version: str = None,
+        only_metadata: bool = False,
         cache_root: str = None,
         num_workers: typing.Optional[int] = 1,
         verbose: bool = True,
@@ -238,6 +238,7 @@ def load_to(
         root: target directory
         name: name of database
         version: version string, latest if ``None``
+        only_metadata: load only metadata
         cache_root: cache folder where databases are stored.
             If not set :meth:`audb.default_cache_root` is used.
             Only used to read the dependencies of the requested version
@@ -253,16 +254,22 @@ def load_to(
     if version is None:
         version = latest_version(name)
 
-    db_root = audeer.safe_path(root)
-    db_root_tmp = database_tmp_folder(db_root)
+    db_root = audeer.path(root)
+    db_root_tmp = database_tmp_root(db_root)
 
     # remove files with a wrong checksum
     # to ensure we load correct version
     update = os.path.exists(db_root) and os.listdir(db_root)
     audeer.mkdir(db_root)
-    deps = dependencies(name, version=version, cache_root=cache_root)
+    deps = dependencies(
+        name,
+        version=version,
+        cache_root=cache_root,
+        verbose=verbose,
+    )
     if update:
-        for file in deps.files:
+        files = deps.tables if only_metadata else deps.files
+        for file in files:
             full_file = os.path.join(db_root, file)
             if os.path.exists(full_file):
                 checksum = audbackend.md5(full_file)
@@ -288,7 +295,10 @@ def load_to(
     # load database
 
     # move header to root and load database ...
-    _move_file(db_root_tmp, db_root, define.HEADER_FILE)
+    audeer.move_file(
+        os.path.join(db_root_tmp, define.HEADER_FILE),
+        os.path.join(db_root, define.HEADER_FILE),
+    )
     try:
         db = audformat.Database.load(
             db_root,
@@ -305,15 +315,19 @@ def load_to(
 
     # get altered and new media files
 
-    media = _find_media(db, db_root, deps, num_workers, verbose)
-    _get_media(media, db_root, db_root_tmp, name, deps, backend,
-               num_workers, verbose)
+    if not only_metadata:
+        media = _find_media(db, db_root, deps, num_workers, verbose)
+        _get_media(media, db_root, db_root_tmp, name, deps, backend,
+                   num_workers, verbose)
 
     # save dependencies
 
     dep_path_tmp = os.path.join(db_root_tmp, define.DEPENDENCIES_FILE)
     deps.save(dep_path_tmp)
-    _move_file(db_root_tmp, db_root, define.DEPENDENCIES_FILE)
+    audeer.move_file(
+        dep_path_tmp,
+        os.path.join(db_root, define.DEPENDENCIES_FILE),
+    )
 
     # save database and remove the temporal directory
     # to signal all files were correctly loaded
@@ -324,7 +338,7 @@ def load_to(
     except OSError:  # pragma: no cover
         raise RuntimeError(
             'Could not remove temporary directory, '
-            'probably there are some leftover files.'
+            'probably there are some leftover files. '
             'This should not happen.'
         )
 
