@@ -14,26 +14,33 @@ import audb
 
 os.environ['AUDB_CACHE_ROOT'] = pytest.CACHE_ROOT
 os.environ['AUDB_SHARED_CACHE_ROOT'] = pytest.SHARED_CACHE_ROOT
-audb.config.REPOSITORIES = pytest.REPOSITORIES
+
+
+@pytest.fixture(
+    scope='session',
+    autouse=True,
+)
+def fixture_set_repositories():
+    audb.config.REPOSITORIES = pytest.REPOSITORIES
 
 
 DB_NAME = f'test_convert-{pytest.ID}'
 DB_ROOT = os.path.join(pytest.ROOT, 'db')
 
 DB_FILES = {
-    os.path.join('audio', 'file1.wav'): {
+    'audio/file1.wav': {
         'bit_depth': 16,
         'channels': 1,
         'format': 'wav',
         'sampling_rate': 8000,
     },
-    os.path.join('audio', 'file2.wav'): {
+    'audio/file2.wav': {
         'bit_depth': 24,
         'channels': 2,
         'format': 'wav',
         'sampling_rate': 16000,
     },
-    os.path.join('audio', 'file3.flac'): {
+    'audio/file3.flac': {
         'bit_depth': 8,
         'channels': 3,
         'format': 'flac',
@@ -43,7 +50,7 @@ DB_FILES = {
 
 
 def clear_root(root: str):
-    root = audeer.safe_path(root)
+    root = audeer.path(root)
     if os.path.exists(root):
         shutil.rmtree(root)
 
@@ -212,50 +219,6 @@ def test_format(format):
             assert converted_file.endswith(format)
 
 
-@pytest.mark.parametrize(
-    'mix',
-    [
-        'mono',
-        'stereo',
-        'left',
-        'right',
-        pytest.param(
-            'left-only',
-            marks=pytest.mark.xfail(raises=ValueError),
-        ),
-        pytest.param(
-            'right-only',
-            marks=pytest.mark.xfail(raises=ValueError),
-        ),
-        None,
-    ],
-)
-def test_mix(mix):
-    # Test for backward compatibility
-    with pytest.warns(UserWarning):
-        db = audb.load(
-            DB_NAME,
-            mix=mix,
-            full_path=False,
-            num_workers=pytest.NUM_WORKERS,
-            verbose=False,
-        )
-    original_files = db['files']['original'].get()
-
-    for converted_file, original_file in zip(db.files, original_files):
-
-        converted_file = os.path.join(db.meta['audb']['root'], converted_file)
-        original_file = os.path.join(DB_ROOT, original_file)
-
-        if mix in ['mono', 'left', 'right']:
-            assert audiofile.channels(converted_file) == 1
-        elif mix == 'stereo':
-            assert audiofile.channels(converted_file) == 2
-        else:
-            assert audiofile.channels(converted_file) == \
-                audiofile.channels(original_file)
-
-
 @pytest.mark.parametrize(
     'mixdown',
     [
